#####################################################################
# 
#   Helper functions used by all pipelines to genereate task list
#
#####################################################################

import os
from helpers import runIDHelpers as rid
from helpers import parameters as params
import pipelineHelpers

##########################################
##########################################
#                                  TASK CREATION                                       #
##########################################
##########################################

#####################################################
#
#  GetProjectNamePath(): Returns <project path>/<name of project>_<runID>
#
#####################################################
def GetProjectNamePathRunID():
   return params.GetProjectPath()  +'/'+ params.GetProjectName() +'_'+ rid.GetRunID() + '/'

#####################################################
#
#  ParseInfile(): return a list of strings, one for each line of the infile, with empty lines removed
#
#####################################################
def ParseInfile(infile):
    
    infile_list=[]
    with open(infile) as f:
        for line in f:
            if line.strip():
                line_list = line.rstrip().split()

                if len(line_list) != 6:
                    log = pipelineHelpers.GetLogFile('MAIN')
                    msg = 'Infile does not have 6 columns in line ' + str(line_list)
                    pipelineHelpers.Logging('ERROR', log, msg)
                    raise Exception(msg)
                
                infile_list.append(line_list)
    
    return infile_list
    

#####################################################
#
#  GetSampleIDs(): Returns tumour, normal ids
#
#####################################################
def GetSampleIDs(job_record):
    return job_record[0]+'_'+job_record[1], job_record[3]+'_'+job_record[4]

#####################################################
#
#  CheckPath(): Check if path exists and create if it does not
#
#####################################################
def CheckPath(path):
    if not os.path.isdir(path):
        os.makedirs(path)
    return path
   
#####################################################
#
#  GetProjectPaths(): get all project paths for a given software dir
#
#####################################################
def GetProjectPaths(software):
    project_name_path = GetProjectNamePathRunID()
    results_path = CheckPath( project_name_path + software + '/')
    intermediate_path = CheckPath( results_path + 'intermediate_' + rid.GetRunID() +'/')
#     plot_path = CheckPath( project_name_path + 'plots/')
    sentinel_path = CheckPath( project_name_path + 'sentinel_' + rid.GetRunID() +'/')
    return (results_path, intermediate_path, sentinel_path)

#####################################################
#
#  CheckSentinel(): Checks if sentinel file exists
#
#####################################################
def CheckSentinel(sentinel_list):
    if len(sentinel_list) != 0:
        for sentinel_file in sentinel_list:
            for sent in sentinel_file:
                if sent[0] != '':
                    if os.path.isfile(sent[0]) != True:
                        return False
    return True


#####################################################
#
#  CreateFileList(): Returns filenames and paths in a list
#
#####################################################
def CreateFileList(file_type, num_files, path):

    job_list = []
    infile_list = ParseInfile(params.GetInfile())

    for line in infile_list:
        file_list = []
        #job_info = line.split()
        
        # Find the tumour_id for the sample
        tumour_id, normal_id = GetSampleIDs(line)
        if file_type == 'bam':
            file_list.append(line[2])
            file_list.append(line[5])
        elif num_files == 1:
            file_list.append(path + file_type.format(tumour_id, normal_id))
        elif num_files >= 2:
            for count in range (1, num_files + 1):
                if num_files < 100:
                    num = str(count).zfill(2)
                else:
                    num = str(count).zfill(3)
                file_list.append(path + file_type.format(tumour_id, normal_id, num))
        else:
            file_list.append('')
        job_list.append(file_list)
    return job_list

########################################################
#
#  CreateTaskList(): Creates a list of tasks by sample

########################################################
def CreateTaskList(inputs, sentinels, outputs, sample_ids, prev_sentinels):
    job_list = []
    count = 0
    
    for sentinel, sample_id in zip(sentinels, sample_ids):
        input_list = []
        output_list = []
        prev_sentinel_list = []
        for inpt in inputs:
            input_list.append(inpt[count])
        for output in outputs:
            output_list.append(output[count])
        for prev_sentinel in prev_sentinels:
            prev_sentinel_list.append(prev_sentinel[count])
            
        mylist = [input_list, sentinel[0], output_list, sample_id[0], prev_sentinel_list]
        job_list.append(mylist)
        count = count + 1
    return job_list

