"""
titanTasks.py
    populates task inputs and outputs

author: @jrosner
"""

import titanHelpers
import taskHelpers
import runIDHelpers as rid

(results_path, intermediate_path,
 sentinel_path) = taskHelpers.GetProjectPaths(titanHelpers.name)


def ReadCountTaskList():
    """populates task inputs and outputs"""

    inputs = []
    outputs = []
    prev_sentinels = []

    prev_sentinels.append(
        taskHelpers.CreateFileList('None', -1, sentinel_path))
    sentinels = taskHelpers.CreateFileList(
        '{0}_{1}_titan_readcount.sentinel', 1, sentinel_path)

    inputs.append(taskHelpers.CreateFileList('bam', 1, intermediate_path))
    outputs.append(taskHelpers.CreateFileList(
        '{0}_{1}_{2}_reads.wig', 2, intermediate_path))
    sample_ids = taskHelpers.CreateFileList('{0}_{1}', 1, '')

    job_parameters = taskHelpers.CreateTaskList(inputs, sentinels, outputs,
                                                sample_ids, prev_sentinels)

    for job in job_parameters:
        yield job


def CorrectReadsTaskList():
    """populates task inputs and outputs"""

    inputs = []
    outputs = []
    prev_sentinels = []

    prev_sentinels.append(taskHelpers.CreateFileList(
        '{0}_{1}_titan_readcount.sentinel', 1, sentinel_path))
    sentinels = taskHelpers.CreateFileList(
        '{0}_{1}_titan_correctreads.sentinel', 1, sentinel_path)

    inputs.append(taskHelpers.CreateFileList(
        '{0}_{1}_{2}_reads.wig', 2, intermediate_path))

    outputs.append(taskHelpers.CreateFileList(
        '{0}_{1}_cn_data.txt', 1, intermediate_path))

    sample_ids = taskHelpers.CreateFileList('{0}_{1}', 1, '')
    job_parameters = taskHelpers.CreateTaskList(
        inputs, sentinels, outputs, sample_ids, prev_sentinels)

    for job in job_parameters:
        yield job


def NormHetPosTaskList():
    """populates task inputs and outputs"""

    inputs = []
    outputs = []
    prev_sentinels = []

    prev_sentinels.append(
        taskHelpers.CreateFileList('None', -1, sentinel_path))
    sentinels = taskHelpers.CreateFileList(
        '{0}_{1}_titan_normalhetpsns.sentinel', 1, sentinel_path)

    inputs.append(taskHelpers.CreateFileList('bam', 1, intermediate_path))
    outputs.append(taskHelpers.CreateFileList(
        '{0}_{1}_normal_hetpsns_chr{2}.txt', 24, intermediate_path))

    sample_ids = taskHelpers.CreateFileList('{0}_{1}', 1, '')

    job_parameters = taskHelpers.CreateTaskList(
        inputs, sentinels, outputs, sample_ids, prev_sentinels)

    for job in job_parameters:
        yield job


def MergeNormChrTaskList():

    inputs = []
    outputs = []
    prev_sentinels = []

    prev_sentinels.append(taskHelpers.CreateFileList(
        '{0}_{1}_titan_normalhetpsns.sentinel', 1, sentinel_path))
    sentinels = taskHelpers.CreateFileList(
        '{0}_{1}_titan_normmerge.sentinel', 1, sentinel_path)

    inputs.append(taskHelpers.CreateFileList(
        '{0}_{1}_normal_hetpsns_chr{2}.txt', 24, intermediate_path))
    outputs.append(taskHelpers.CreateFileList(
        '{0}_{1}_normal_counts_hetpsns.txt', 1, intermediate_path))

    sample_ids = taskHelpers.CreateFileList('{0}_{1}', 1, '')

    job_parameters = taskHelpers.CreateTaskList(
        inputs, sentinels, outputs, sample_ids, prev_sentinels)

    for job in job_parameters:
        yield job


def TumourAllelicCountTaskList():
    """populates task inputs and outputs"""

    inputs = []
    outputs = []
    prev_sentinels = []

    prev_sentinels.append(taskHelpers.CreateFileList(
        '{0}_{1}_titan_normalhetpsns.sentinel', 1, sentinel_path))
    sentinels = taskHelpers.CreateFileList(
        '{0}_{1}_titan_tumourcounts.sentinel', 1, sentinel_path)

    inputs.append(taskHelpers.CreateFileList('bam', 1, intermediate_path))
    inputs.append(taskHelpers.CreateFileList(
        '{0}_{1}_normal_hetpsns_chr{2}.txt', 24, intermediate_path))
    outputs.append(taskHelpers.CreateFileList(
        '{0}_{1}_tumour_counts_hetpsns_chr{2}.txt', 24, intermediate_path))

    sample_ids = taskHelpers.CreateFileList('{0}_{1}', 1, '')

    job_parameters = taskHelpers.CreateTaskList(
        inputs, sentinels, outputs, sample_ids, prev_sentinels)

    for job in job_parameters:
        yield job


def FilterTumourAllelicCountTaskList():
    """populates task inputs and outputs"""

    inputs = []
    outputs = []
    prev_sentinels = []

    prev_sentinels.append(taskHelpers.CreateFileList(
        '{0}_{1}_titan_tumourcounts.sentinel', 1, sentinel_path))
    sentinels = taskHelpers.CreateFileList(
        '{0}_{1}_titan_filteredcounts.sentinel', 1, sentinel_path)

    inputs.append(taskHelpers.CreateFileList(
        '{0}_{1}_tumour_counts_hetpsns_chr{2}.txt', 24, intermediate_path))
    outputs.append(taskHelpers.CreateFileList(
        '{0}_{1}_tumour_counts_hetpsns_chr{2}.filtered.txt', 24, intermediate_path))

    sample_ids = taskHelpers.CreateFileList('{0}_{1}', 1, '')

    job_parameters = taskHelpers.CreateTaskList(
        inputs, sentinels, outputs, sample_ids, prev_sentinels)

    for job in job_parameters:
        yield job


def MergeTumChrTaskList():
    """populates task inputs and outputs"""

    inputs = []
    outputs = []
    prev_sentinels = []

    prev_sentinels.append(taskHelpers.CreateFileList(
        '{0}_{1}_titan_filteredcounts.sentinel', 1, sentinel_path))
    sentinels = taskHelpers.CreateFileList(
        '{0}_{1}_titan_tummerge.sentinel', 1, sentinel_path)

    inputs.append(taskHelpers.CreateFileList(
        '{0}_{1}_tumour_counts_hetpsns_chr{2}.filtered.txt', 24, intermediate_path))
    outputs.append(taskHelpers.CreateFileList(
        '{0}_{1}_tumour_counts_hetpsns.txt', 1, intermediate_path))

    sample_ids = taskHelpers.CreateFileList('{0}_{1}', 1, '')

    job_parameters = taskHelpers.CreateTaskList(
        inputs, sentinels, outputs, sample_ids, prev_sentinels)

    for job in job_parameters:
        yield job


def RunTitanTaskList():
    """populates task inputs and outputs"""

    inputs = []
    outputs = []
    prev_sentinels = []

    prev_sentinels.append(taskHelpers.CreateFileList(
        '{0}_{1}_titan_tummerge.sentinel', 1, sentinel_path))
    prev_sentinels.append(taskHelpers.CreateFileList(
        '{0}_{1}_titan_correctreads.sentinel', 1, sentinel_path))
    sentinels = taskHelpers.CreateFileList(
        '{0}_{1}_titan_titan.sentinel', 1, sentinel_path)

    inputs.append(taskHelpers.CreateFileList(
        '{0}_{1}_tumour_counts_hetpsns.txt', 1, intermediate_path))
    inputs.append(taskHelpers.CreateFileList(
        '{0}_{1}_cn_data.txt', 1, intermediate_path))

    num_clusters = int(titanHelpers.GetNumClusters())
    for i in range(1, num_clusters + 1):
        file_str = '{0}_{1}_' + rid.GetRunID() + '_cluster_' + str(i)
        outputs.append(taskHelpers.CreateFileList(
            file_str + '_titan.txt', 1, results_path))
        outputs.append(taskHelpers.CreateFileList(
            file_str + '_params.txt', 1, results_path))

    sample_ids = taskHelpers.CreateFileList('{0}_{1}', 1, '')
    job_parameters = taskHelpers.CreateTaskList(
        inputs, sentinels, outputs, sample_ids, prev_sentinels)

    for job in job_parameters:
        yield job


def PlotTitanTaskList():
    """populates task inputs and outputs"""

    inputs = []
    outputs = []
    prev_sentinels = []

    prev_sentinels.append(taskHelpers.CreateFileList(
        '{0}_{1}_titan_titan.sentinel', 1, sentinel_path))
    sentinels = taskHelpers.CreateFileList(
        '{0}_{1}_titan_plot.sentinel', 1, sentinel_path)

    num_clusters = int(titanHelpers.GetNumClusters())
    for i in range(1, num_clusters + 1):
        file_str = '{0}_{1}_' + \
            rid.GetRunID() + '_cluster_' + str(i) + '.RData'
        inputs.append(taskHelpers.CreateFileList(file_str, 1, results_path))
        outputs.append(taskHelpers.CreateFileList('', 1, results_path))

    sample_ids = taskHelpers.CreateFileList('{0}_{1}', 1, '')
    job_parameters = taskHelpers.CreateTaskList(inputs, sentinels, outputs,
                                                sample_ids, prev_sentinels)

    for job in job_parameters:
        yield job


def CreateSegFilesTaskList():
    """populates task inputs and outputs"""

    inputs = []
    outputs = []
    prev_sentinels = []

    prev_sentinels.append(taskHelpers.CreateFileList(
        '{0}_{1}_titan_titan.sentinel', 1, sentinel_path))
    sentinels = taskHelpers.CreateFileList(
        '{0}_{1}_titan_createSegFiles.sentinel', 1, sentinel_path)

    num_clusters = int(titanHelpers.GetNumClusters())
    for i in range(1, num_clusters + 1):
        file_str = '{0}_{1}_' + rid.GetRunID() + '_cluster_' + str(i)
        inputs.append(taskHelpers.CreateFileList(
            file_str + '_titan.txt', 1, results_path))
        outputs.append(taskHelpers.CreateFileList(
            file_str + '_segs.txt', 1, results_path))
        outputs.append(taskHelpers.CreateFileList(
            file_str + '.seg', 1, results_path))

    sample_ids = taskHelpers.CreateFileList('{0}_{1}', 1, '')
    job_parameters = taskHelpers.CreateTaskList(
        inputs, sentinels, outputs, sample_ids, prev_sentinels)

    for job in job_parameters:
        yield job
