"""
filterCounts.py
    filter positions from input file that are not found in the dbSNP positions file
    input is assumed to be a single chromosome

author:@jrosner
"""

import argparse
import pysam

#---------------------------------------------
#  Main Program
#---------------------------------------------
if __name__ == '__main__':

    chrom_list = map(str, range(1, 23)) + ['X', 'Y']

    # command line arguments
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-t', '--tumour', action='store', dest='tumour_file', required=True,
        help='tumour allelic counts file')
    parser.add_argument(
        '-r', '--reference', action='store', dest='ref_file', required=True,
        help='reference vcf file')
    parser.add_argument(
        '-c', '--chromosome', action='store', dest='chrom', choices=chrom_list, default='',
        help='chromosome (all if not defined)')
    parser.add_argument(
        '-o', '--outfile', action='store', dest='outfile', required=True,
        help='filtered output file')
    args = parser.parse_args()

    # read tumour file into dictionary
    # store the line and a boolean that denotes whether or not that position
    # is in the reference
    calls = {}
    tumour_file = open(args.tumour_file, 'r')
    for line in tumour_file:
        t_chrom = line.split()[0]
        t_pos = int(line.split()[1])
        calls[(t_chrom, t_pos)] = [line, False]

    tumour_file.close()

    # scan vcf reference file -- any positions that exist in the calls will be
    # flagged true
    tabix_file = pysam.Tabixfile(args.ref_file)
    for record in tabix_file.fetch(args.chrom):
        if (record.split()[0], int(record.split()[1])) in calls:
            calls[(record.split()[0], int(record.split()[1]))][1] = True

    # filter false calls, sort into an iterable and write to outfile
    outfile = open(args.outfile, 'w')
    for k, v in sorted([(k, v) for k, v in calls.iteritems() if v[1]]):
        outfile.write(v[0])

    outfile.close()
