args <- commandArgs(TRUE)

library(TitanCNA)
obj_file <- args[1]
outdir <- args[2]
numClusters <- args[3]
id <- args[4]
rid <- args[5]

chr <- c(1:22,"X")

# load a workspace into the current session
load(obj_file)

#### PLOT RESULTS ####
dir.create(paste(outdir,"/",id,"_",rid,"_cluster_",numClusters,"/",sep=""))
library(SNPchip)  ## use this library to plot chromosome idiogram (optional)
norm <- tail(convergeParams$n,1) #convergeParams$n[length(convergeParams$n)]
ploidy <- tail(convergeParams$phi,1) #convergeParams$phi[length(convergeParams$phi)]
for (i in chr){
    outplot <- paste(outdir,"/",id,"_",rid,"_cluster_",numClusters,"/",id,"_",rid,"_cluster_",numClusters,"_chr",i,".png",sep="")
    png(outplot,width=1200,height=1000,res=100,type="cairo")
    
    # if 2 or fewer clusters, use c(4,1) panels
    if (as.numeric(numClusters) <= 2){
		par(mfrow=c(4,1))
	}else{
		par(mfrow=c(3,1))  
	}
    
    ## PLOT LOG RATIO (CNA) ##
    plotCNlogRByChr(results, i, ploidy=ploidy, geneAnnot=NULL, spacing=4, 
    				ylim=c(-4,6), cex=0.5, main=paste("Chr ",i,sep=""), xlab="")
    
    ## PLOT ALLELIC RATIOS (LOH) ##
    plotAllelicRatio(results, i, geneAnnot=NULL, spacing=4, 
    				ylim=c(0,1), cex=0.5, main=paste("Chr ",i,sep=""), xlab="")
    				
    ## PLOT CELLULAR PREVALENCE AND CLONAL CLUSTERS ##
    plotClonalFrequency(results, i, normal=norm, geneAnnot=NULL, spacing=4,
                    ylim=c(0,1), cex=0.5, main=paste("Chr ",i,sep=""), xlab="")
    
    ## PLOT SUBCLONE PROFILE FOR 2 OR FEWER CLONAL CLUSTER RUNS ##
    if (as.numeric(numClusters) <= 2){
		plotSubcloneProfiles(results, i, cex = 2, spacing=6, main=paste("Chr ",i,sep=""))
		pI <- plotIdiogram(i,build="hg19",unit="bp",label.y=-4.25,new=FALSE,ylim=c(-2,-1))
	}else{
		pI <- plotIdiogram(i,build="hg19",unit="bp",label.y=-0.35,new=FALSE,ylim=c(-0.2,-0.1))
	}
	
    dev.off()
}
