'''
Created on 2012-01-17

@author: Andrew Roth
'''
import os
from libc.stdlib cimport malloc, free

cdef class FastaFile:
    def __cinit__(self, char * file_name):
        self._file_name = strdup(file_name)
        
        self._fasta_file = fai_load(file_name)

        if self._fasta_file == NULL:
            raise Exception("Could not open FASTA file {0}".format(file_name))
        
        index_file_name = file_name + ".fai"
        
        if not os.path.exists(index_file_name):
            raise Exception("Index not found for FASTA file {0}".format(file_name))

    def __dealloc__(self):
        if self._fasta_file != NULL:
            fai_destroy(self._fasta_file)
            self._fasta_file = NULL
        
        if self._file_name != NULL:
            free(self._file_name)
    
    def get_base(self, reference, position):
        return self.get_reference_base(reference, position)
     
    cdef int get_slice(self, char * reference, char * _s, int start, int end):
        #print "\tpopulating slice:", start, end
        for i in range(0, end - start):
            _s[i] = self.get_reference_base(reference, start + i)
        return 0

    def homopolymer_forward(self, contig, position):
        return self.count_run_forward(contig, position, 0, 1)

    def homopolymer_backward(self, contig, position):
        #print "\ttrace:", self.get_reference_base(reference, position), self.get_reference_base(reference, position - 1),
        return self.count_run_backward(contig, position, 0, 1)

    cdef faidx_t * get_file_pointer(self):
        return self._fasta_file

    def n_mer_repeat(self, reference, position, n):
        count = 1
        return self.count_run_forward(reference, position * n, count, n)
        
    cdef int count_run_forward(self, char * contig, int position, int count, int n):
        count += 1
        if count >= 100:
            return count
        if int(self.get_reference_base(contig, position)) == int(self.get_reference_base(contig, position + 1)):
            return self.count_run_forward(contig, position + 1, count, 1)
        return count

    cdef int count_run_backward(self, char * contig, int position, int count, int n):
        count += 1
        if count >= 100:
            return count
        if int(self.get_reference_base(contig, position)) == int(self.get_reference_base(contig, position - 1)):
            return self.count_run_backward(contig, position - 1, count, 1)
        return count
            

    cdef int _polymer(self, char * reference, int position, int _n):
        #print "entering polymer"
        n = abs(_n)
        #print "sizeof:", sizeof(char) * n
        home_slice = <char *> malloc(sizeof(char) * n)
        slice_buffer = <char *> malloc(sizeof(char) * n)
        count = 0
        
        if _n > 0:
            self.get_slice(reference, home_slice, position, position + n)
            self.get_slice(reference, slice_buffer, position + ((count + 1) * n), position + ((count + 2) * n))
        else:
            self.get_slice(reference, home_slice, position - n + 1, position + 1)
            self.get_slice(reference, slice_buffer, position - ((count + 2) * n) + 1, position - ((count + 1) * n) + 1)

        while(self.compare_slice(n, home_slice, slice_buffer) and count < 10):
            #print "home: \"" + home_slice + "\"\t\"" + slice_buffer + "\""
            count += 1
            if _n > 0:
                self.get_slice(reference, slice_buffer, position + ((count + 1) * n), position + ((count + 2) * n))
            else:
                self.get_slice(reference, slice_buffer, position - ((count + 2) * n) + 1, position - ((count + 1) * n) + 1)

        free(slice_buffer)
        free(home_slice)
        return count

    cdef int compare_slice(self, int n, char * first, char * second):
        for i in range(0, n):
            if first[i] != second[i]:
                return 0
        return 1

    cdef char get_reference_base(self, char * reference, int position):
        '''
        Retrieve a base form the file. 
        
        position is 1 based.
        '''     
        cdef char ref_base
        cdef int len
        
        len = 1
        
        ref_base = self._fetch(reference, position, position + 1, & len)[0]
        
        ref_base = < char > toupper(< int > ref_base)

        return ref_base   

    # THIS IS THE ONLY PLACE WHERE ZERO INDEXED POSITIONS ARE EVEN TO BE THOUGHT ABOUT!
    cdef char * _fetch(self, char * reference, int start, int end, int * length):
        return faidx_fetch_seq(self._fasta_file, reference, start - 1, end, length)
    
